// Generated by CoffeeScript 2.3.2
(function() {
  var Category, File, Promise, Router, Site, Tag, URL, chokidar, colors, default404, fse, getContentType, getFileLayout, getPathFn, getURLFn, getVersion, http, isArray, isCurrentPathFn, isFunction, isObject, isString, matchFiles, moment, parseFrontMatter, path, yaml;

  fse = require("fs-extra");

  path = require("path");

  yaml = require("js-yaml");

  http = require("http");

  ({URL} = require("url"));

  colors = require("colors/safe");

  moment = require("moment-timezone");

  chokidar = require("chokidar");

  Promise = require("bluebird");

  ({Site, File, Category, Tag} = require("./types"));

  ({isArray, isString, isFunction, isObject, default404, matchFiles, getVersion, getPathFn, getURLFn, getContentType, getFileLayout, isCurrentPathFn, parseFrontMatter} = require("./utils"));

  Router = class Router {
    constructor(logger, renderer, processor, generator, translator, site) {
      this.loadFile = this.loadFile.bind(this);
      this.saveFile = this.saveFile.bind(this);
      this.loadLanguage = this.loadLanguage.bind(this);
      this.loadContext = this.loadContext.bind(this);
      this.matchAll = this.matchAll.bind(this);
      this.buildServerRoutes = this.buildServerRoutes.bind(this);
      this.watchAll = this.watchAll.bind(this);
      this.unwatchAll = this.unwatchAll.bind(this);
      this.handleEvents = this.handleEvents.bind(this);
      this.listen = this.listen.bind(this);
      this.handle = this.handle.bind(this);
      this.build = this.build.bind(this);
      this.serve = this.serve.bind(this);
      this.logger = logger;
      this.renderer = renderer;
      this.processor = processor;
      this.generator = generator;
      this.translator = translator;
      this.site = site;
      this._ = {};
      this.watchers = [];
      this.watchedEvents = [];
      this.sourcePages = [];
      this.handling = false;
      this.getURL = getURLFn(this.site["siteConfig"]["baseURL"], this.site["siteConfig"]["rootDir"]);
      this.getPath = getPathFn(this.site["siteConfig"]["rootDir"]);
      moment.locale(this.site["siteConfig"]["language"]);
    }

    read(path) {
      return fse.readFile(path);
    }

    write(content, file) {
      if (!file["isBinary"]) {
        return fse.outputFile(path.join(file["docDir"], file["docPath"]), content);
      }
      return fse.copy(path.join(file["srcDir"], file["srcPath"]), path.join(file["docDir"], file["docPath"]));
    }

    async loadFile(file) {
      var j, len, raw, result, results, results1;
      this.logger.debug(`Hikaru is reading \`${colors.cyan(path.join(file["srcDir"], file["srcPath"]))}\`...`);
      raw = (await this.read(path.join(file["srcDir"], file["srcPath"])));
      // Auto detect if a file is a binary file or a UTF-8 encoding text file.
      if (raw.equals(Buffer.from(raw.toString("utf8"), "utf8"))) {
        raw = raw.toString("utf8");
        file["isBinary"] = false;
      } else {
        file["isBinary"] = true;
      }
      file["raw"] = raw;
      file["text"] = raw;
      file = parseFrontMatter(file);
      results = (await Promise.all(this.renderer.render(file)));
      results1 = [];
      for (j = 0, len = results.length; j < len; j++) {
        result = results[j];
        if (isFunction(result["content"])) {
          result["type"] = "template";
          results1.push(this.site["templates"][path.basename(result["srcPath"], path.extname(result["srcPath"]))] = result["content"]);
        } else if (result["layout"] === "post") {
          result["type"] = "post";
          results1.push(this.site.put("posts", result));
        } else if (result["layout"] != null) {
          result["type"] = "page";
          results1.push(this.site.put("pages", result));
        } else {
          result["type"] = "asset";
          results1.push(this.site.put("assets", result));
        }
      }
      return results1;
    }

    async saveFile(file) {
      var layout;
      this.logger.debug(`Hikaru is writing \`${colors.cyan(path.join(file["docDir"], file["docPath"]))}\`...`);
      layout = getFileLayout(file, Object.keys(this.site["templates"]));
      if (layout != null) {
        return this.write((await this.site["templates"][layout](this.loadContext(file))), file);
      } else {
        return this.write(file["content"], file);
      }
    }

    loadLanguage(file) {
      var err, lang, language;
      lang = file["language"] || this.site["siteConfig"]["language"];
      if (!(lang in this.translator.list())) {
        try {
          language = yaml.safeLoad(fse.readFileSync(path.join(this.site["siteConfig"]["themeDir"], "languages", `${lang}.yml`)));
          this.translator.register(lang, language);
        } catch (error) {
          err = error;
          if (err["code"] === "ENOENT") {
            this.logger.warn(`Hikaru cannot find \`${lang}\` language file in your theme.`);
          }
        }
      }
      return lang;
    }

    loadContext(file) {
      var lang;
      lang = this.loadLanguage(file);
      return Object.assign(new File(), file, {
        "site": this.site,
        "siteConfig": this.site["siteConfig"],
        "themeConfig": this.site["themeConfig"],
        "moment": moment,
        "getVersion": getVersion,
        "getURL": this.getURL,
        "getPath": this.getPath,
        "isCurrentPath": isCurrentPathFn(this.site["siteConfig"]["rootDir"], file["docPath"]),
        "isArray": isArray,
        "isString": isString,
        "isFunction": isFunction,
        "isObject": isObject,
        "__": this.translator.getTranslateFn(lang)
      });
    }

    async matchAll() {
      return ((await matchFiles(path.join("**", "*"), {
        "nodir": true,
        "dot": true,
        "cwd": this.site["siteConfig"]["themeSrcDir"]
      }))).map((srcPath) => {
        return new File(this.site["siteConfig"]["docDir"], this.site["siteConfig"]["themeSrcDir"], srcPath);
      }).concat(((await matchFiles(path.join("**", "*"), {
        "nodir": true,
        "dot": true,
        "cwd": this.site["siteConfig"]["srcDir"]
      }))).map((srcPath) => {
        return new File(this.site["siteConfig"]["docDir"], this.site["siteConfig"]["srcDir"], srcPath);
      }));
    }

    buildServerRoutes(allFiles) {
      var f, j, key, len, results1;
      this._ = {};
      results1 = [];
      for (j = 0, len = allFiles.length; j < len; j++) {
        f = allFiles[j];
        key = this.getPath(f["docPath"]);
        this.logger.debug(`Hikaru is serving \`${colors.cyan(key)}\`...`);
        results1.push(this._[key] = f);
      }
      return results1;
    }

    watchAll() {
      var j, len, ref, results1, srcDir;
      ref = [this.site["siteConfig"]["themeSrcDir"], this.site["siteConfig"]["srcDir"]];
      results1 = [];
      for (j = 0, len = ref.length; j < len; j++) {
        srcDir = ref[j];
        results1.push(((srcDir) => {
          var event, k, len1, ref1, results2, watcher;
          watcher = chokidar.watch(path.join("**", "*"), {
            "cwd": srcDir,
            "ignoreInitial": true
          });
          this.watchers.push(watcher);
          ref1 = ["add", "change", "unlink"];
          results2 = [];
          for (k = 0, len1 = ref1.length; k < len1; k++) {
            event = ref1[k];
            results2.push(((event) => {
              return watcher.on(event, (srcPath) => {
                var i;
                this.logger.debug(`Hikaru watched event \`${colors.blue(event)}\` from \`${colors.cyan(path.join(srcDir, srcPath))}\``);
                i = this.watchedEvents.findIndex(function(p) {
                  return p["srcDir"] === srcDir && p["srcPath"] === srcPath;
                });
                if (i !== -1) {
                  // Just update event.
                  this.watchedEvents[i]["type"] = event;
                } else {
                  // Not found.
                  this.watchedEvents.push({
                    "type": event,
                    "srcDir": srcDir,
                    "srcPath": srcPath
                  });
                }
                return setImmediate(this.handleEvents);
              });
            })(event));
          }
          return results2;
        })(srcDir));
      }
      return results1;
    }

    unwatchAll() {
      var results1, w;
      results1 = [];
      while ((w = this.watchers.shift()) != null) {
        results1.push(w.close());
      }
      return results1;
    }

    async handleEvents() {
      var e, file, j, key, len, ref;
      // Keep handling atomic. Prevent repeatedly handling.
      if (!this.watchedEvents.length || this.handling) {
        return;
      }
      this.handling = true;
      while ((e = this.watchedEvents.shift()) != null) {
        file = new File(this.site["siteConfig"]["docDir"], e["srcDir"], e["srcPath"]);
        if (e["type"] === "unlink") {
          ref = ["assets", "pages", "posts"];
          for (j = 0, len = ref.length; j < len; j++) {
            key = ref[j];
            this.site.del(key, file);
          }
        } else {
          file = (await this.loadFile(file));
        }
      }
      await this.handle();
      this.buildServerRoutes(this.site["assets"].concat(this.site["posts"]).concat(this.site["pages"]).concat(this.site["files"]));
      return this.handling = false;
    }

    listen(ip, port) {
      var server;
      server = http.createServer(async(request, response) => {
        var layout, res, url;
        // Remove query string.
        url = request["url"].split(/[?#]/)[0];
        if (!(url in this._)) {
          this.logger.log(`404: ${url}`);
          res = this._[this.getPath("404.html")] || {
            "content": default404,
            "docPath": this.getPath("404.html")
          };
          response.writeHead(404, {
            "Content-Type": getContentType(res["docPath"])
          });
        } else {
          this.logger.log(`200: ${url}`);
          res = this._[url];
          response.writeHead(200, {
            "Content-Type": getContentType(res["docPath"])
          });
        }
        layout = getFileLayout(res, Object.keys(this.site["templates"]));
        if (layout != null) {
          response.write((await this.site["templates"][layout](this.loadContext(res))));
        } else {
          response.write(res["content"]);
        }
        return response.end();
      });
      process.prependListener("exit", () => {
        server.close();
        this.logger.log(`Hikaru stopped listening on http://${ip}:${port}${this.getPath()}...`);
        return this.unwatchAll();
      });
      this.logger.log(`Hikaru is listening on http://${ip}:${port}${this.getPath()}...`);
      if (ip !== "localhost") {
        server.listen(port, ip);
      } else {
        server.listen(port);
      }
      return this.watchAll();
    }

    async handle() {
      this.site = (await this.processor.process(this.site));
      return this.site["files"] = (await this.generator.generate(this.site));
    }

    async build() {
      await Promise.all(((await this.matchAll())).map(this.loadFile));
      await this.handle();
      return this.site["assets"].concat(this.site["posts"]).concat(this.site["pages"]).concat(this.site["files"]).map(this.saveFile);
    }

    async serve(ip, port) {
      await Promise.all(((await this.matchAll())).map(this.loadFile));
      await this.handle();
      this.buildServerRoutes(this.site["assets"].concat(this.site["posts"]).concat(this.site["pages"]).concat(this.site["files"]));
      return this.listen(ip, port);
    }

  };

  module.exports = Router;

}).call(this);
